#!/usr/bin/env python

# TODO fails with renaming a service on a hostgroup, Geomap places service on host in config gui so the hg-connection never really exists

from __future__ import print_function
from lxml import etree
from nacoma.hooks import Change
import sys

target = "/opt/monitor/op5/ninja/application/config/geomap/locations.xml"

def handle(xml, change):
    any_changes = 0
    if change.type not in ['host', 'service', 'hostgroup', 'servicegroup']:
        # basic whitelisting in order to concat the type safely into
        # ElementPath query
        return any_changes

    # we're looping instead of adding the attribute directly into the search
    # above, to not have to bother with objects being renamed to characters not
    # suitable for xpath strings. correct this if you're certain about this
    # domain; I'm not
    elements = xml.findall(".//"+change.type)
    if change.type == 'host':
        elements += xml.findall(".//service") # we need to rename services parent pointer, the 'host' attribute

    if not elements:
        return any_changes
    for element in elements:
        if change.type == 'service' and change.oldname == element.attrib['host']+";"+element.attrib['description']:
            if change.is_renamed():
                element.attrib['host'], element.attrib['description'] = change.newname.split(';')
                any_changes = 1
            elif change.is_deleted():
                element.getparent().remove(element)
                any_changes = 1
        elif change.type == 'host' and element.tag == 'service' and element.attrib['host'] == change.oldname:
            if change.is_renamed():
                element.attrib['host'] = change.newname
                any_changes = 1
            elif change.is_deleted():
                element.getparent().remove(element)
                any_changes = 1
        elif change.type in ['host', 'servicegroup', 'hostgroup'] and element.tag == change.type and change.oldname == element.attrib['name']:
            if change.is_renamed():
                element.attrib['name'] = change.newname
                any_changes = 1
            elif change.is_deleted():
                element.getparent().remove(element)
                any_changes = 1
    return any_changes

def main(xml_filename):
    try:
        with open(xml_filename, 'r') as filehandle:
            locations_xml = etree.parse(filehandle)
    except IOError:
        print("Not updating geomap locations: could not read '%s'" % xml_filename, file=sys.stderr)
        sys.exit(1)
    changes = 0
    for line in sys.stdin:
        changes |= handle(locations_xml, Change(line))
    if changes:
        try:
            with open(xml_filename, 'w') as filehandle:
                locations_xml.write(filehandle)
        except IOError:
            print("Not updating geomap locations: could not write to '%s'" % xml_filename, file=sys.stderr)
            sys.exit(1)


if __name__ == "__main__":
    main(target)
